def form_created_message(form_id: str):
    print(f"Form created successfully: {form_id}")


def form_updated_message(form_id: str):
    print(f"Form updated successfully: {form_id}")


def not_implemented(*args, **kwargs):
    print("Invalid command")


def required_argument(argument):
    print(f"required argument: [{argument}]")


def required_parameter(parameter):
    print(f"required parameter: --{parameter} [{parameter}]")


def invalid_variable(argument):
    print(f"invalid variable: {argument}")


def invalid_parameter(parameter):
    print(f"invalid parameter: --{parameter} [{parameter}]")


def invalid_flag_parameter_value(parameter):
    print(
        f"invalid parameter value: --{parameter} [{parameter}] value must be true or false"
    )


def invalid_non_flag_parameter_value(parameter):
    print(f"invalid parameter value: --{parameter} [{parameter}] value must be string")


def invalid_background_parameter_value():
    print(
        f"""
        invalid parameter value: --background [background] value must be a color or image path.\n
        Allowed image extensions are: [.png, .jpeg, .jpg, .svg]"""
    )


def duplicate_variable(argument):
    print(f"duplicate variable: {argument}")


def code_and_file_not_allowed():
    print("you can set either code or file parameters, but not both.")


def missing_parameters_to_update(form_id):
    print(f"missing parameters to be updated of form {form_id}")


def file_path_does_not_exists_message(path):
    print(f"file path not found: {path}")


def error_upload_background_message(path):
    print(f"Some error happened during background upload: {path}")



import json
import requests
import urllib.request
import urllib.response

from .utils_config import get_auth_info, get_credentials

HACKERFORMS_API_URL = "https://hackerforms-api.abstra.cloud"
HACKERFORMS_HASURA_URL = "https://hackerforms-hasura.abstra.cloud/v1/graphql"
ABSTRA_ASSETS_UPLOAD_URL = "https://upload.abstra.cloud"


def hf_api_runner(method, path, data=None):
    api_token, workspace_id, _ = get_auth_info()
    response = requests.request(
        method,
        f"{HACKERFORMS_API_URL}/workspaces/{workspace_id}/{path}",
        data=json.dumps(data) if data else None,
        headers={"content-type": "application/json", "API-Authorization": api_token},
    )
    return response.json()


def upload_file(filepath, file):
    response_json = hf_api_runner("POST", "put-url", {"filepath": filepath})
    req = urllib.request.Request(
        url=response_json["putURL"], method="PUT", data=file.read()
    )
    res = urllib.request.urlopen(req)
    return res.status < 400


def asset_upload(filepath, file):
    response = requests.request(
        "POST",
        f"{ABSTRA_ASSETS_UPLOAD_URL}/asset/",
        headers={
            "cache-control": "no-cache",
            "Pragma": "no-cache",
            "content-type": "application/json",
        },
        data=json.dumps({"filepath": filepath}),
    )

    response_json = response.json()
    try:

        req = urllib.request.Request(
            url=response_json["putURL"], method="PUT", data=file
        )
        res = urllib.request.urlopen(req)
        if res.status < 400:
            return response_json["getURL"]
    except Exception as e:
        print(e)

    raise Exception("Some error ocurred in asset upload")


def get_file_signed_url(filepath):
    response_json = hf_api_runner("POST", "get-url", {"filepath": filepath})
    return response_json.get("getURL")


def list_workspace_files():
    return hf_api_runner("GET", "files")


def delete_file(filepath):
    return hf_api_runner("DELETE", "file", {"filepath": filepath})


def hf_hasura_runner(query, variables={}):
    api_token = get_credentials()
    response = requests.post(
        HACKERFORMS_HASURA_URL,
        data=json.dumps({"query": query, "variables": variables}),
        headers={"content-type": "application/json", "API-Authorization": api_token},
    )
    if response.status_code >= 300:
        raise Exception(f"Request error: {response.text}")
    jsond = response.json()
    if "data" in jsond:
        return jsond["data"]
    return jsond["errors"]


def list_workspace_packages():
    query = """
        query GetPackages {
            packages {
                name
                version
            }
        }
    """
    return hf_hasura_runner(query).get("packages", [])


def list_workspace_forms():
    query = """
        query GetForms {
            forms {
                id
                title
            }
        }
    """
    return hf_hasura_runner(query).get("forms", [])


def list_workspace_vars():
    query = """
        query GetVars {
            environment_variables {
                name
                value
            }
        }
    """
    return hf_hasura_runner(query).get("environment_variables", [])


def get_subdomain_by_form_id(form_id: str):
    query = """
        query SubdomainByFormId($form_id: uuid!) {
            subdomains(where: {workspace: {forms: {id: {_eq: $form_id}}}}) {
                name
            }
        }
    """

    return hf_hasura_runner(query, {"form_id": form_id}).get("subdomains", [])


def add_workspace_vars(raw_vars):
    _, workspace_id, _ = get_auth_info()
    vars = [
        {"name": v["name"], "value": v["value"], "workspace_id": workspace_id}
        for v in raw_vars
    ]
    query = """
        mutation InsertVars($vars: [environment_variables_insert_input!]!) {
            insert_environment_variables(
                objects: $vars
                on_conflict: {
                    constraint: environment_variables_name_workspace_id_key
                    update_columns: [value, name]
                }
            ) {
                returning {
                    name
                    value
                }
            }
        }
    """
    return (
        hf_hasura_runner(query, {"vars": vars})
        .get("insert_environment_variables", {})
        .get("returning", [])
    )


def add_workspace_packages(raw_packages):
    _, workspace_id, _ = get_auth_info()
    packages = [
        {"name": p["name"], "version": p["version"], "workspace_id": workspace_id}
        for p in raw_packages
    ]
    query = """
        mutation InsertPackages($packages: [packages_insert_input!]!) {
            insert_packages(
                objects: $packages
                on_conflict: {
                    constraint: packages_workspace_id_name_key  
                    update_columns: [version]
                }
            ) {
                returning {
                    name
                    version
                }
            }
        }
    """
    return (
        hf_hasura_runner(query, {"packages": packages})
        .get("insert_packages", {})
        .get("returning", [])
    )


def add_workspace_form(data):
    _, workspace_id, _ = get_auth_info()
    form_data = {
        "title": data["name"],
        "workspace_id": workspace_id,
        "script": {
            "data": {
                "code": data["code"],
                "workspace_id": workspace_id,
                "name": data["name"],
            }
        },
    }

    data.pop("name")
    data.pop("code")
    form_data.update(data)

    query = """
        mutation InsertForm($form_data: [forms_insert_input!]!) {
            insert_forms(
                objects: $form_data
            ) {
                returning {
                    id
                    title
                    script {
                        id
                        code
                    }
                }
            }
        }
    """
    return (
        hf_hasura_runner(query, {"form_data": form_data})
        .get("insert_forms", {})
        .get("returning", {})[0]
    )


def update_workspace_form(form_id, data):
    form_data = {}
    script_data = {}
    name = data.get("name", None)
    if name:
        data.pop("name")
        form_data["title"] = name
        script_data["name"] = name

    code = data.get("code", None)
    if code:
        data.pop("code")
        if "data" in script_data:
            script_data["code"] = code
        else:
            script_data = {}
            script_data["code"] = code

    for param, value in data.items():
        form_data[param] = value

    request_data = {"id": form_id, "form_data": form_data}

    form_query = """
        mutation UpdateForm($id: uuid!, $form_data: forms_set_input, $script_data: scripts_set_input = {}) {
            update_forms_by_pk(pk_columns: {id: $id}, _set: $form_data) {
                id
                title
                script {
                    id
                    code
                }
            }
            $script_mutation
        }
    """

    if code or name:
        script = """
            update_scripts(where: {form: {id: {_eq: $id}}}, _set: $script_data) {
                returning {
                    name
                    code
                }
            }
        """
        form_query = form_query.replace("$script_mutation", script)
        request_data["script_data"] = script_data
    else:
        form_query = form_query.replace("$script_mutation", "")

    return hf_hasura_runner(form_query, request_data).get("update_forms_by_pk", {})


def delete_workspace_packages(packages):
    query = """
        mutation DeletePackages($packages: [String!]) {
            delete_packages(where: {name: {_in: $packages}}) {
                returning {
                    name
                    version
                }
            }
        }
    """
    return (
        hf_hasura_runner(query, {"packages": packages})
        .get("delete_packages", {})
        .get("returning", [])
    )


def delete_workspace_vars(vars):
    query = """
        mutation DeleteVars($vars: [String!]) {
            delete_environment_variables(where: {name: {_in: $vars}}) {
                returning {
                    name
                    value
                }
            }
        }
    """
    return (
        hf_hasura_runner(query, {"vars": vars})
        .get("delete_environment_variables", {})
        .get("returning", [])
    )


def delete_workspace_form(form_id):
    query = """
    mutation DeleteForm($id: uuid!) {
        delete_forms_by_pk(id: $id) {
            id
        }
    }
    """

    return hf_hasura_runner(query, {"id": form_id})



from abstra_cli.utils import parse_package
from abstra_cli.cli_helpers import print_packages
from abstra_cli.apis import (
    add_workspace_packages,
    list_workspace_packages,
    delete_workspace_packages,
)
from abstra_cli.resources.resources import Resource


class Packages(Resource):
    @staticmethod
    def list():
        packages = list_workspace_packages()
        print_packages(packages)

    @staticmethod
    def add(*args, **kwargs):
        packages = list(args)
        file = (
            kwargs.get("f")
            or kwargs.get("file")
            or kwargs.get("r")
            or kwargs.get("requirement")
        )
        if file:
            with open(file, "r") as f:
                packages.extend([p for p in f.read().split("\n") if p])

        processed_packages = []
        processed_names = []
        for pkg in packages:
            name, version = parse_package(pkg)
            if not name:
                print(f"Invalid package: {pkg}")
                return False
            if name in processed_names:
                print(f"Duplicate package: {pkg}")
                return False
            processed_packages.append({"name": name, "version": version})

        added_packages = add_workspace_packages(processed_packages)
        print_packages(added_packages)
        print(f"\nAdded {len(added_packages)} packages")

    @staticmethod
    def remove(*args, **kwargs):
        deleted_packages = delete_workspace_packages(args)
        print_packages(deleted_packages)
        print(f"\nDeleted {len(deleted_packages)} packages")



from progress.bar import FillingSquaresBar

from .utils import digits, format_digits


def read_credentials():
    print(
        "Abstra API Tokens can be found in your workspace or at https://forms.abstra.run/737986ce-a8ed-4c7b-bd7e-5f0b11331b66."
    )
    credentials = input(f"API Token: ")
    if not credentials:
        raise Exception("No API token configured")
    return credentials


def show_progress(message, max) -> FillingSquaresBar:
    return FillingSquaresBar(message, suffix="%(percent)d%%", max=max)


def print_files(files):
    files.sort(key=lambda x: x["Key"])
    max_d = digits(max([f["Size"] for f in files]))
    for file in files:
        print(
            f"{format_digits(file['Size'], max_d)} - {file['LastModified']}: {file['Key']}"
        )


def print_vars(vars):
    vars.sort(key=lambda x: x["name"])
    for var in vars:
        print(f"{var['name']}={var['value']}")


def print_packages(packages):
    packages.sort(key=lambda x: x["name"])
    for pkg in packages:
        print(f"{pkg['name']}{'==' + pkg['version'] if pkg['version'] else ''}")


def print_forms(forms):
    forms.sort(key=lambda x: x["title"])
    for form in forms:
        print(f"{form['id']} {form['title']}")



import os
from .public_apis import get_info_from_token, usage

ABSTRA_FOLDER = ".abstra/"
CREDENTIALS_FILE = ".abstra/credentials"


def create_abstra_dir():
    if not os.path.exists(ABSTRA_FOLDER):
        os.makedirs(ABSTRA_FOLDER)


def save_credentials(api_token: str):
    create_abstra_dir()
    with open(CREDENTIALS_FILE, "w") as f:
        f.write(api_token.strip())


def get_credentials():
    if os.getenv("ABSTRA_API_TOKEN"):
        return os.getenv("ABSTRA_API_TOKEN")

    if not os.path.exists(CREDENTIALS_FILE):
        return None

    with open(CREDENTIALS_FILE) as f:
        return f.read().strip()


def get_auth_info():
    api_token = get_credentials()
    if not api_token:
        return None, None, None
    workspace_id, author_id = get_info_from_token(api_token)
    return api_token, workspace_id, author_id


def credentials_check(f):
    def wrapper(*args, **kwargs):
        api_token, workspace_id, author_id = get_auth_info()

        if not api_token:
            raise Exception("No API token configured")

        if not workspace_id:
            raise Exception("Bad token: no workspace found")

        usage(f, args, kwargs, author_id, workspace_id)
        return f(*args, **kwargs)

    return wrapper


def configuration_check(f):
    def wrapper(*args, **kwargs):
        api_token, workspace_id, author_id = get_auth_info()
        usage(f, args, kwargs, author_id, workspace_id)
        return f(*args, **kwargs)

    return wrapper



import webbrowser

from abstra_cli.resources.resources import Resource
from ..apis import (
    add_workspace_form,
    list_workspace_forms,
    delete_workspace_form,
    update_workspace_form,
    asset_upload,
    get_subdomain_by_form_id,
)
from ..cli_helpers import print_forms
from ..messages import (
    code_and_file_not_allowed,
    invalid_parameter,
    required_argument,
    required_parameter,
    invalid_flag_parameter_value,
    invalid_non_flag_parameter_value,
    missing_parameters_to_update,
    invalid_background_parameter_value,
    file_path_does_not_exists_message,
    error_upload_background_message,
    form_created_message,
    form_updated_message,
)

from ..utils import (
    check_color,
    check_is_image_path,
    slugify_filename,
    path_exists,
    get_prod_form_url,
)

NAME_PARAMETERS = ["name"]
CODE_PARAMETERS = ["file", "f", "code", "c"]
BACKGROUND_PARAMETERS = ["background"]
FLAG_PARAMETERS = ["auto_start", "allow_restart", "show_sidebar"]
OTHER_PARAMETERS = [
    "start_message",
    "end_message",
    "error_message",
    "timeout_mesage",
    "main_color",
    "start_button_text",
    "logo_url",
    "log_messages",
    "font_color",
    "welcome_title",
    "brand_name",
]
NON_FLAG_PARAMETERS = (
    NAME_PARAMETERS + CODE_PARAMETERS + BACKGROUND_PARAMETERS + OTHER_PARAMETERS
)

FORM_PARAMETERS = FLAG_PARAMETERS + NON_FLAG_PARAMETERS


def evaluate_parameter_name(parameters, form_data: dict) -> dict:
    name = parameters.get("name") or parameters.get("n")
    if not name:
        required_parameter("name")
        exit()

    form_data["name"] = name
    return form_data


def add_parameters_file_and_code(parameters: dict, form_data: dict) -> dict:
    EMPTY_FORM = "from hackerforms import *"
    file = parameters.get("file") or parameters.get("f")
    code = parameters.get("code") or parameters.get("c")

    if file and code:
        code_and_file_not_allowed()
        exit()

    if file:
        with open(file, "r") as f:
            form_data["code"] = f.read()
            return form_data

    if code:
        form_data["code"] = code
        return form_data

    form_data["code"] = EMPTY_FORM
    return form_data


def update_parameters_file_and_code(parameters, form_data):
    file = parameters.get("file") or parameters.get("f")
    code = parameters.get("code") or parameters.get("c")

    if file and code:
        code_and_file_not_allowed()
        exit()

    if file:
        with open(file, "r") as f:
            form_data["code"] = f.read()

    if code:
        form_data["code"] = code

    return form_data


def build_other_parameters(parameters, form_data, valid_parameters):
    for param, value in parameters.items():
        if param in valid_parameters:
            form_data[param] = value

    return form_data


def check_valid_parameters(parameters, valid_parameters):
    for param in parameters.keys():
        if param not in valid_parameters:
            invalid_parameter(param)
            exit()


def evaluate_non_flag_parameters_values(parameters, form_data, non_flag_parameters):
    for param, value in parameters.items():
        if param in non_flag_parameters and value in [True, False]:
            invalid_non_flag_parameter_value(param)
            exit()

    return form_data


def evaluate_flag_parameters(parameters, form_data, flag_parameters):
    for param, value in parameters.items():
        if param in flag_parameters:
            if value == "true" or value == True:
                form_data[param] = True
                continue
            if value == "false" or value == False:
                form_data[param] = False
                continue

            invalid_flag_parameter_value(param)
            exit()

    return form_data


def evaluate_background_parameter_value(parameters: dict, form_data: dict):
    background = parameters.get("background", None)
    if not background:
        return

    if check_color(background):
        form_data["theme"] = background
        return form_data

    if not path_exists(background):
        file_path_does_not_exists_message(background)
        exit()

    if check_is_image_path(background):
        filename = slugify_filename(background)
        try:
            with open(background, "rb") as f:
                file = f.read()
                url = asset_upload(filename, file)
                form_data["theme"] = url
                return form_data
        except Exception as e:
            error_upload_background_message(background)
            exit()

    invalid_background_parameter_value()
    exit()


class Forms(Resource):
    @staticmethod
    def list():
        forms = list_workspace_forms()
        print_forms(forms)

    @staticmethod
    def add(*args, **kwargs):
        form_data = {}
        check_valid_parameters(kwargs, FORM_PARAMETERS)
        form_data = evaluate_parameter_name(kwargs, form_data.copy())
        form_data = add_parameters_file_and_code(kwargs, form_data.copy())
        form_data = evaluate_flag_parameters(kwargs, form_data.copy(), FLAG_PARAMETERS)
        form_data = evaluate_non_flag_parameters_values(
            kwargs, form_data.copy(), NON_FLAG_PARAMETERS
        )
        form_data = build_other_parameters(kwargs, form_data.copy(), OTHER_PARAMETERS)
        form_data = evaluate_background_parameter_value(kwargs, form_data.copy())
        if form_data:
            form_id = add_workspace_form(form_data)["id"]
        form_created_message(form_id)

    @staticmethod
    def update(*args, **kwargs):

        if not len(args):
            required_argument("form_id")
            exit()

        form_id = args[0]
        if not len(kwargs):
            missing_parameters_to_update(form_id)
            exit()

        form_data = {}
        check_valid_parameters(kwargs, FORM_PARAMETERS)
        form_data = update_parameters_file_and_code(kwargs, form_data.copy())
        form_data = evaluate_flag_parameters(kwargs, form_data.copy(), FLAG_PARAMETERS)
        form_data = evaluate_non_flag_parameters_values(
            kwargs, form_data.copy(), NON_FLAG_PARAMETERS
        )
        form_data = build_other_parameters(
            kwargs, form_data.copy(), OTHER_PARAMETERS + NAME_PARAMETERS
        )
        form_data = evaluate_background_parameter_value(kwargs, form_data.copy())
        if form_data:
            update_workspace_form(form_id, form_data)
        form_updated_message(form_id)

    @staticmethod
    def remove(*args, **kwargs):
        if not len(args):
            required_argument("form_id")
            exit()
        form_id = args[0]
        delete_workspace_form(form_id)

    @staticmethod
    def play(*args, **kwargs):
        if not len(args):
            required_argument("form_id")
            exit()
        form_id = args[0]
        response = get_subdomain_by_form_id(form_id)
        if not len(response):
            print(
                "There is no workspace related to this form id. Please, verify whether it is correct."
            )
            exit()
        subdomain_name = response[0]["name"]
        url = get_prod_form_url(subdomain_name, form_id)
        webbrowser.open(url)



import os
import json
import requests
from functools import lru_cache


@lru_cache(maxsize=None)
def get_info_from_token(api_token):
    response = requests.get(
        f"https://auth.abstra.cloud/abstra-cloud",
        headers={"content-type": "application/json", "API-Authorization": api_token},
    )
    response_json = response.json()
    author_id = response_json.get("author_id")
    workspaces = response_json.get("workspaces", [{}])

    workspace_id = None
    if len(workspaces) != 0:
        workspace_id = workspaces[0].get("id")

    return workspace_id, author_id


def usage(f, args, kwargs, author_id, workspace_id):
    if os.getenv("DISABLE_USAGE_STATISTICS"):
        return

    try:
        requests.post(
            "https://usage-api.abstra.cloud/api/rest/cli-usage",
            data=json.dumps(
                {
                    "author_id": author_id,
                    "workspace_id": workspace_id,
                    "method": f.__name__,
                    "arguments": {"args": args[1:], "kwargs": list(kwargs.keys())},
                }
            ),
            headers={"content-type": "application/json"},
        )
    except Exception as e:
        pass
